@@ -63,6 +63,9 @@ SCHEMAS = [
         schema_id="chrome://browser/content/asrouter/schemas/MessagingExperiment.schema.json",
         schema_path=Path("MessagingExperiment.schema.json"),
         message_types={
+            "BookmarksBarButton": (
+                SCHEMA_DIR / "OnboardingMessage" / "BookmarksBarButton.schema.json"
+            ),
             "CFRUrlbarChiclet": (
                 SCHEMA_DIR / "CFR" / "templates" / "CFRUrlbarChiclet.schema.json"
             ),
@@ -70,9 +73,15 @@ SCHEMAS = [
                 SCHEMA_DIR / "CFR" / "templates" / "ExtensionDoorhanger.schema.json"
             ),
             "InfoBar": SCHEMA_DIR / "CFR" / "templates" / "InfoBar.schema.json",
+            "MenuMessage": (
+                SCHEMA_DIR / "OnboardingMessage" / "MenuMessage.schema.json"
+            ),
             "NewtabPromoMessage": (
                 SCHEMA_DIR / "PBNewtab" / "NewtabPromoMessage.schema.json"
             ),
+            "NewtabMessage": (
+                SCHEMA_DIR / "OnboardingMessage" / "NewtabMessage.schema.json"
+            ),
             "Spotlight": SCHEMA_DIR / "OnboardingMessage" / "Spotlight.schema.json",
             "ToastNotification": (
                 SCHEMA_DIR / "ToastNotification" / "ToastNotification.schema.json"
