@@ -6,10 +6,12 @@
 
 ChromeUtils.defineESModuleGetters(this, {
   QuickSuggest: "resource:///modules/QuickSuggest.sys.mjs",
+  SearchUtils: "moz-src:///toolkit/components/search/SearchUtils.sys.mjs",
   TelemetryTestUtils: "resource://testing-common/TelemetryTestUtils.sys.mjs",
   UrlbarProviderAutofill: "resource:///modules/UrlbarProviderAutofill.sys.mjs",
   UrlbarProviderQuickSuggest:
     "resource:///modules/UrlbarProviderQuickSuggest.sys.mjs",
+  UrlbarSearchUtils: "resource:///modules/UrlbarSearchUtils.sys.mjs",
 });
 
 add_setup(async function setUpQuickSuggestXpcshellTest() {
@@ -17,432 +19,9 @@ add_setup(async function setUpQuickSuggestXpcshellTest() {
   // jumping through a bunch of hoops. Suggest's use of TelemetryEnvironment is
   // tested in browser tests, and there's no other necessary reason to wait for
   // TelemetryEnvironment initialization in xpcshell tests, so just skip it.
-  UrlbarPrefs._testSkipTelemetryEnvironmentInit = true;
+  QuickSuggest._testSkipTelemetryEnvironmentInit = true;
 });
 
-let gAddTasksWithRustSetup;
-
-/**
- * Adds two tasks: One with the Rust backend disabled and one with it enabled.
- * The names of the task functions will be the name of the passed-in task
- * function appended with "_rustDisabled" and "_rustEnabled". If the passed-in
- * task doesn't have a name, "anonymousTask" will be used.
- *
- * Call this with the usual `add_task()` arguments. Additionally, an object with
- * the following properties can be specified as any argument:
- *
- * {boolean} skip_if_rust_enabled
- *   If true, a "_rustEnabled" task won't be added. Useful when Rust is enabled
- *   by default but the task doesn't make sense with Rust and you still want to
- *   test some behavior when Rust is disabled.
- *
- * @param {...any} args
- *   The usual `add_task()` arguments.
- */
-function add_tasks_with_rust(...args) {
-  let skipIfRustEnabled = false;
-  let i = args.findIndex(a => a.skip_if_rust_enabled);
-  if (i >= 0) {
-    skipIfRustEnabled = true;
-    args.splice(i, 1);
-  }
-
-  let taskFnIndex = args.findIndex(a => typeof a == "function");
-  let taskFn = args[taskFnIndex];
-
-  for (let rustEnabled of [false, true]) {
-    let newTaskName =
-      (taskFn.name || "anonymousTask") +
-      (rustEnabled ? "_rustEnabled" : "_rustDisabled");
-
-    if (rustEnabled && skipIfRustEnabled) {
-      info(
-        "add_tasks_with_rust: Skipping due to skip_if_rust_enabled: " +
-          newTaskName
-      );
-      continue;
-    }
-
-    let newTaskFn = async (...taskFnArgs) => {
-      info("add_tasks_with_rust: Setting rustEnabled: " + rustEnabled);
-      UrlbarPrefs.set("quicksuggest.rustEnabled", rustEnabled);
-      info("add_tasks_with_rust: Done setting rustEnabled: " + rustEnabled);
-
-      // The current backend may now start syncing, so wait for it to finish.
-      info("add_tasks_with_rust: Forcing sync");
-      await QuickSuggestTestUtils.forceSync();
-      info("add_tasks_with_rust: Done forcing sync");
-
-      if (gAddTasksWithRustSetup) {
-        info("add_tasks_with_rust: Calling setup function");
-        await gAddTasksWithRustSetup();
-        info("add_tasks_with_rust: Done calling setup function");
-      }
-
-      let rv;
-      try {
-        info(
-          "add_tasks_with_rust: Calling original task function: " + taskFn.name
-        );
-        rv = await taskFn(...taskFnArgs);
-      } catch (e) {
-        // Clearly report any unusual errors to make them easier to spot and to
-        // make the flow of the test clearer. The harness throws NS_ERROR_ABORT
-        // when a normal assertion fails, so don't report that.
-        if (e.result != Cr.NS_ERROR_ABORT) {
-          Assert.ok(
-            false,
-            "add_tasks_with_rust: The original task function threw an error: " +
-              e
-          );
-        }
-        throw e;
-      } finally {
-        info(
-          "add_tasks_with_rust: Done calling original task function: " +
-            taskFn.name
-        );
-        info("add_tasks_with_rust: Clearing rustEnabled");
-        UrlbarPrefs.clear("quicksuggest.rustEnabled");
-        info("add_tasks_with_rust: Done clearing rustEnabled");
-
-        // The current backend may now start syncing, so wait for it to finish.
-        info("add_tasks_with_rust: Forcing sync");
-        await QuickSuggestTestUtils.forceSync();
-        info("add_tasks_with_rust: Done forcing sync");
-      }
-      return rv;
-    };
-
-    Object.defineProperty(newTaskFn, "name", { value: newTaskName });
-
-    let addTaskArgs = [];
-    for (let j = 0; j < args.length; j++) {
-      addTaskArgs[j] =
-        j == taskFnIndex
-          ? newTaskFn
-          : Cu.cloneInto(args[j], this, { cloneFunctions: true });
-    }
-    add_task(...addTaskArgs);
-  }
-}
-
-/**
- * Registers a setup function that `add_tasks_with_rust()` will await before
- * calling each of your original tasks. Call this at most once in your test file
- * (i.e., in `add_setup()`). This is useful when enabling/disabling Rust has
- * side effects related to your particular test that need to be handled or
- * awaited for each of your tasks. On the other hand, if only one or two of your
- * tasks need special setup, do it directly in those tasks instead of using
- * this.
- *
- * @param {Function} setupFn
- *   A function that will be awaited before your original tasks are called.
- */
-function registerAddTasksWithRustSetup(setupFn) {
-  gAddTasksWithRustSetup = setupFn;
-}
-
-/**
- * Returns an expected Wikipedia (non-sponsored) result that can be passed to
- * `check_results()` regardless of whether the Rust backend is enabled.
- *
- * @returns {object}
- *   An object that can be passed to `check_results()`.
- */
-function makeWikipediaResult({
-  source,
-  provider,
-  keyword = "wikipedia",
-  title = "Wikipedia Suggestion",
-  url = "http://example.com/wikipedia",
-  originalUrl = "http://example.com/wikipedia",
-  icon = null,
-  iconBlob = new Blob([new Uint8Array([])]),
-  impressionUrl = "http://example.com/wikipedia-impression",
-  clickUrl = "http://example.com/wikipedia-click",
-  blockId = 2,
-  advertiser = "Wikipedia",
-  iabCategory = "5 - Education",
-  suggestedIndex = -1,
-  isSuggestedIndexRelativeToGroup = true,
-}) {
-  let result = {
-    suggestedIndex,
-    isSuggestedIndexRelativeToGroup,
-    type: UrlbarUtils.RESULT_TYPE.URL,
-    source: UrlbarUtils.RESULT_SOURCE.SEARCH,
-    heuristic: false,
-    payload: {
-      title,
-      url,
-      originalUrl,
-      displayUrl: url.replace(/^https:\/\//, ""),
-      isSponsored: false,
-      qsSuggestion: keyword,
-      sponsoredAdvertiser: "Wikipedia",
-      sponsoredIabCategory: "5 - Education",
-      isBlockable: true,
-      blockL10n: {
-        id: "urlbar-result-menu-dismiss-icecat-suggest",
-      },
-      isManageable: true,
-      telemetryType: "adm_nonsponsored",
-    },
-  };
-
-  if (UrlbarPrefs.get("quickSuggestRustEnabled")) {
-    result.payload.source = source || "rust";
-    result.payload.provider = provider || "Wikipedia";
-    result.payload.iconBlob = iconBlob;
-  } else {
-    result.payload.source = source || "remote-settings";
-    result.payload.provider = provider || "AdmWikipedia";
-    result.payload.icon = icon;
-    result.payload.sponsoredImpressionUrl = impressionUrl;
-    result.payload.sponsoredClickUrl = clickUrl;
-    result.payload.sponsoredBlockId = blockId;
-    result.payload.sponsoredAdvertiser = advertiser;
-    result.payload.sponsoredIabCategory = iabCategory;
-  }
-
-  return result;
-}
-
-/**
- * Returns an expected AMP (sponsored) result that can be passed to
- * `check_results()` regardless of whether the Rust backend is enabled.
- *
- * @returns {object}
- *   An object that can be passed to `check_results()`.
- */
-function makeAmpResult({
-  source,
-  provider,
-  keyword = "amp",
-  title = "Amp Suggestion",
-  url = "http://example.com/amp",
-  originalUrl = "http://example.com/amp",
-  icon = null,
-  iconBlob = new Blob([new Uint8Array([])]),
-  impressionUrl = "http://example.com/amp-impression",
-  clickUrl = "http://example.com/amp-click",
-  blockId = 1,
-  advertiser = "Amp",
-  iabCategory = "22 - Shopping",
-  suggestedIndex = -1,
-  isSuggestedIndexRelativeToGroup = true,
-  requestId = undefined,
-} = {}) {
-  let result = {
-    suggestedIndex,
-    isSuggestedIndexRelativeToGroup,
-    type: UrlbarUtils.RESULT_TYPE.URL,
-    source: UrlbarUtils.RESULT_SOURCE.SEARCH,
-    heuristic: false,
-    payload: {
-      title,
-      url,
-      originalUrl,
-      requestId,
-      displayUrl: url.replace(/^https:\/\//, ""),
-      isSponsored: true,
-      qsSuggestion: keyword,
-      sponsoredImpressionUrl: impressionUrl,
-      sponsoredClickUrl: clickUrl,
-      sponsoredBlockId: blockId,
-      sponsoredAdvertiser: advertiser,
-      sponsoredIabCategory: iabCategory,
-      isBlockable: true,
-      blockL10n: {
-        id: "urlbar-result-menu-dismiss-icecat-suggest",
-      },
-      isManageable: true,
-      telemetryType: "adm_sponsored",
-      descriptionL10n: { id: "urlbar-result-action-sponsored" },
-    },
-  };
-
-  if (UrlbarPrefs.get("quickSuggestRustEnabled")) {
-    result.payload.source = source || "rust";
-    result.payload.provider = provider || "Amp";
-    if (result.payload.source == "rust") {
-      result.payload.iconBlob = iconBlob;
-    } else {
-      result.payload.icon = icon;
-    }
-  } else {
-    result.payload.source = source || "remote-settings";
-    result.payload.provider = provider || "AdmWikipedia";
-    result.payload.icon = icon;
-  }
-
-  return result;
-}
-
-/**
- * Returns an expected MDN result that can be passed to `check_results()`
- * regardless of whether the Rust backend is enabled.
- *
- * @returns {object}
- *   An object that can be passed to `check_results()`.
- */
-function makeMdnResult({ url, title, description }) {
-  let finalUrl = new URL(url);
-  finalUrl.searchParams.set("utm_medium", "icecat-desktop");
-  finalUrl.searchParams.set("utm_source", "icecat-suggest");
-  finalUrl.searchParams.set(
-    "utm_campaign",
-    "icecat-mdn-web-docs-suggestion-experiment"
-  );
-  finalUrl.searchParams.set("utm_content", "treatment");
-
-  let result = {
-    isBestMatch: true,
-    suggestedIndex: 1,
-    type: UrlbarUtils.RESULT_TYPE.URL,
-    source: UrlbarUtils.RESULT_SOURCE.OTHER_NETWORK,
-    heuristic: false,
-    payload: {
-      telemetryType: "mdn",
-      title,
-      url: finalUrl.href,
-      originalUrl: url,
-      displayUrl: finalUrl.href.replace(/^https:\/\//, ""),
-      description,
-      icon: "chrome://global/skin/icons/mdn.svg",
-      shouldShowUrl: true,
-      bottomTextL10n: { id: "icecat-suggest-mdn-bottom-text" },
-    },
-  };
-
-  if (UrlbarPrefs.get("quickSuggestRustEnabled")) {
-    result.payload.source = "rust";
-    result.payload.provider = "Mdn";
-  } else {
-    result.payload.source = "remote-settings";
-    result.payload.provider = "MDNSuggestions";
-  }
-
-  return result;
-}
-
-/**
- * Returns an expected AMO (addons) result that can be passed to
- * `check_results()` regardless of whether the Rust backend is enabled.
- *
- * @returns {object}
- *   An object that can be passed to `check_results()`.
- */
-function makeAmoResult({
-  source,
-  provider,
-  title = "Amo Suggestion",
-  description = "Amo description",
-  url = "http://example.com/amo",
-  originalUrl = "http://example.com/amo",
-  icon = null,
-  setUtmParams = true,
-}) {
-  if (setUtmParams) {
-    url = new URL(url);
-    url.searchParams.set("utm_medium", "icecat-desktop");
-    url.searchParams.set("utm_source", "icecat-suggest");
-    url = url.href;
-  }
-
-  let result = {
-    isBestMatch: true,
-    suggestedIndex: 1,
-    type: UrlbarUtils.RESULT_TYPE.URL,
-    source: UrlbarUtils.RESULT_SOURCE.SEARCH,
-    heuristic: false,
-    payload: {
-      source,
-      provider,
-      title,
-      description,
-      url,
-      originalUrl,
-      icon,
-      displayUrl: url.replace(/^https:\/\//, ""),
-      shouldShowUrl: true,
-      bottomTextL10n: { id: "icecat-suggest-addons-recommended" },
-      helpUrl: QuickSuggest.HELP_URL,
-      telemetryType: "amo",
-    },
-  };
-
-  if (UrlbarPrefs.get("quickSuggestRustEnabled")) {
-    result.payload.source = source || "rust";
-    result.payload.provider = provider || "Amo";
-  } else {
-    result.payload.source = source || "remote-settings";
-    result.payload.provider = provider || "AddonSuggestions";
-  }
-
-  return result;
-}
-
-/**
- * Returns an expected weather result that can be passed to `check_results()`
- * regardless of whether the Rust backend is enabled.
- *
- * @returns {object}
- *   An object that can be passed to `check_results()`.
- */
-function makeWeatherResult({
-  source,
-  provider,
-  telemetryType = undefined,
-  temperatureUnit = undefined,
-} = {}) {
-  if (!temperatureUnit) {
-    temperatureUnit =
-      Services.locale.regionalPrefsLocales[0] == "en-US" ? "f" : "c";
-  }
-
-  let result = {
-    type: UrlbarUtils.RESULT_TYPE.DYNAMIC,
-    source: UrlbarUtils.RESULT_SOURCE.SEARCH,
-    heuristic: false,
-    suggestedIndex: 1,
-    payload: {
-      temperatureUnit,
-      url: MerinoTestUtils.WEATHER_SUGGESTION.url,
-      iconId: "6",
-      requestId: MerinoTestUtils.server.response.body.request_id,
-      source: "merino",
-      provider: "accuweather",
-      dynamicType: "weather",
-      city: MerinoTestUtils.WEATHER_SUGGESTION.city_name,
-      temperature:
-        MerinoTestUtils.WEATHER_SUGGESTION.current_conditions.temperature[
-          temperatureUnit
-        ],
-      currentConditions:
-        MerinoTestUtils.WEATHER_SUGGESTION.current_conditions.summary,
-      forecast: MerinoTestUtils.WEATHER_SUGGESTION.forecast.summary,
-      high: MerinoTestUtils.WEATHER_SUGGESTION.forecast.high[temperatureUnit],
-      low: MerinoTestUtils.WEATHER_SUGGESTION.forecast.low[temperatureUnit],
-      shouldNavigate: true,
-    },
-  };
-
-  if (UrlbarPrefs.get("quickSuggestRustEnabled")) {
-    result.payload.source = source || "rust";
-    result.payload.provider = provider || "Weather";
-    if (telemetryType !== null) {
-      result.payload.telemetryType = telemetryType || "weather";
-    }
-  } else {
-    result.payload.source = source || "merino";
-    result.payload.provider = provider || "accuweather";
-  }
-
-  return result;
-}
-
 /**
  * Tests quick suggest prefs migrations.
  *
@@ -461,11 +40,11 @@ function makeWeatherResult({
  *     default-branch values. These should be the default prefs for the given
  *     `migrationVersion` and will be set as defaults before migration occurs.
  *
- * @param {string} options.scenario
- *   The scenario to set at the time migration occurs.
  * @param {object} options.expectedPrefs
  *   The expected prefs after migration: `{ defaultBranch, userBranch }`
  *   Pref names should be relative to `browser.urlbar`.
+ * @param {boolean} options.shouldEnable
+ *   Whether Suggest should be enabled when migration occurs.
  * @param {object} [options.initialUserBranch]
  *   Prefs to set on the user branch before migration ocurs. Use these to
  *   simulate user actions like disabling prefs or opting in or out of the
@@ -473,8 +52,8 @@ function makeWeatherResult({
  */
 async function doMigrateTest({
   testOverrides,
-  scenario,
   expectedPrefs,
+  shouldEnable = true,
   initialUserBranch = {},
 }) {
   info(
@@ -482,7 +61,7 @@ async function doMigrateTest({
       JSON.stringify({
         testOverrides,
         initialUserBranch,
-        scenario,
+        shouldEnable,
         expectedPrefs,
       })
   );
@@ -527,9 +106,7 @@ async function doMigrateTest({
   let userBranch = Services.prefs.getBranch("browser.urlbar.");
 
   // Set initial prefs. `initialDefaultBranch` are icecat.js values, i.e.,
-  // defaults immediately after startup and before any scenario update and
-  // migration happens.
-  UrlbarPrefs._updatingIceCatSuggestScenario = true;
+  // defaults immediately after startup, before Suggest init and migration.
   UrlbarPrefs.clear("quicksuggest.migrationVersion");
   let initialDefaultBranch = {
     "suggest.quicksuggest.nonsponsored": false,
@@ -549,19 +126,18 @@ async function doMigrateTest({
       }
     }
   }
-  UrlbarPrefs._updatingIceCatSuggestScenario = false;
 
-  // Update the scenario and check prefs twice. The first time the migration
+  // Reinitialize Suggest and check prefs twice. The first time the migration
   // should happen, and the second time the migration should not happen and
   // all the prefs should stay the same.
   for (let i = 0; i < 2; i++) {
-    info(`Calling updateIceCatSuggestScenario, i=${i}`);
+    info(`Reinitializing Suggest, i=${i}`);
 
-    // Do the scenario update and set `isStartup` to simulate startup.
-    await UrlbarPrefs.updateIceCatSuggestScenario({
+    // Reinitialize Suggest.
+    await QuickSuggest._test_reinit({
       ...testOverrides,
-      scenario,
-      isStartup: true,
+      region: shouldEnable ? "US" : "XX",
+      locale: shouldEnable ? "en-US" : "xx-XX",
     });
 
     // Check expected pref values. Store expected effective values as we go so
@@ -585,7 +161,7 @@ async function doMigrateTest({
       }
 
       info(
-        `Checking expected prefs on ${branchType} branch after updating scenario`
+        `Checking expected prefs on ${branchType} branch after Suggest init`
       );
       for (let [name, value] of entries) {
         expectedEffectivePrefs[name] = value;
@@ -626,7 +202,7 @@ async function doMigrateTest({
 
     let currentVersion =
       testOverrides?.migrationVersion === undefined
-        ? UrlbarPrefs.ICECAT_SUGGEST_MIGRATION_VERSION
+        ? QuickSuggest.MIGRATION_VERSION
         : testOverrides.migrationVersion;
     Assert.equal(
       UrlbarPrefs.get("quicksuggest.migrationVersion"),
@@ -636,7 +212,6 @@ async function doMigrateTest({
   }
 
   // Clean up.
-  UrlbarPrefs._updatingIceCatSuggestScenario = true;
   UrlbarPrefs.clear("quicksuggest.migrationVersion");
   let userBranchNames = [
     ...Object.keys(initialUserBranch),
@@ -645,19 +220,238 @@ async function doMigrateTest({
   for (let name of userBranchNames) {
     userBranch.clearUserPref(name);
   }
-  UrlbarPrefs._updatingIceCatSuggestScenario = false;
+}
+
+/**
+ * Does a test that dismisses a single result by triggering a command on it.
+ *
+ * @param {object} options
+ *   Options object.
+ * @param {SuggestFeature} options.feature
+ *   The feature that provides the dismissed result.
+ * @param {UrlbarResult} options.result
+ *   The result to trigger the command on.
+ * @param {string} options.command
+ *   The name of the command to trigger. It should dismiss one result.
+ * @param {Array} options.queriesForDismissals
+ *   Array of objects: `{ query, expectedResults }`
+ *   For each object, the test will perform a search with `query` as the search
+ *   string. After dismissing the result, the query shouldn't match any results.
+ *   After clearing dismissals, the query should match the results in
+ *   `expectedResults`. If `expectedResults` is omitted, `[result]` will be
+ *   used.
+ * @param {Array} options.queriesForOthers
+ *   Array of objects: `{ query, expectedResults }`
+ *   For each object, the test will perform a search with `query` as the search
+ *   string. The query should always match `expectedResults`.
+ */
+async function doDismissOneTest({
+  feature,
+  result,
+  command,
+  queriesForDismissals,
+  queriesForOthers,
+}) {
+  await QuickSuggest.clearDismissedSuggestions();
+  await QuickSuggestTestUtils.forceSync();
+  Assert.ok(
+    !(await QuickSuggest.canClearDismissedSuggestions()),
+    "Sanity check: canClearDismissedSuggestions should return false initially"
+  );
+
+  let changedPromise = TestUtils.topicObserved(
+    "quicksuggest-dismissals-changed"
+  );
+
+  triggerCommand({
+    result,
+    command,
+    feature,
+    expectedCountsByCall: {
+      removeResult: 1,
+    },
+  });
+
+  info("Awaiting dismissals-changed promise");
+  await changedPromise;
+
+  Assert.ok(
+    await QuickSuggest.canClearDismissedSuggestions(),
+    "canClearDismissedSuggestions should return true after triggering command"
+  );
+  Assert.ok(
+    await QuickSuggest.isResultDismissed(result),
+    "The result should be dismissed"
+  );
+
+  for (let { query } of queriesForDismissals) {
+    info("Doing search for dismissed suggestions: " + JSON.stringify(query));
+    await check_results({
+      context: createContext(query, {
+        providers: [UrlbarProviderQuickSuggest.name],
+        isPrivate: false,
+      }),
+      matches: [],
+    });
+  }
+
+  for (let { query, expectedResults } of queriesForOthers) {
+    info(
+      "Doing search for non-dismissed suggestions: " + JSON.stringify(query)
+    );
+    await check_results({
+      context: createContext(query, {
+        providers: [UrlbarProviderQuickSuggest.name],
+        isPrivate: false,
+      }),
+      matches: expectedResults,
+    });
+  }
+
+  let clearedPromise = TestUtils.topicObserved(
+    "quicksuggest-dismissals-cleared"
+  );
+
+  info("Clearing dismissals");
+  await QuickSuggest.clearDismissedSuggestions();
+
+  // It's not necessary to await this -- awaiting `clearDismissedSuggestions()`
+  // is sufficient -- but we do it to make sure the notification is sent.
+  info("Awaiting dismissals-cleared promise");
+  await clearedPromise;
+
+  Assert.ok(
+    !(await QuickSuggest.canClearDismissedSuggestions()),
+    "canClearDismissedSuggestions should return false after clearing dismissals"
+  );
+
+  for (let { query, expectedResults = [result] } of queriesForDismissals) {
+    info("Doing search after clearing dismissals: " + JSON.stringify(query));
+    await check_results({
+      context: createContext(query, {
+        providers: [UrlbarProviderQuickSuggest.name],
+        isPrivate: false,
+      }),
+      matches: expectedResults,
+    });
+  }
+}
+
+/**
+ * Does a test that dismisses a suggestion type (i.e., all suggestions of a
+ * certain type) by triggering a command on a result.
+ *
+ * @param {object} options
+ *   Options object.
+ * @param {SuggestFeature} options.feature
+ *   The feature that provides the suggestion type.
+ * @param {UrlbarResult} options.result
+ *   The result to trigger the command on.
+ * @param {string} options.command
+ *   The name of the command to trigger. It should dismiss all results of a
+ *   suggestion type.
+ * @param {string} options.pref
+ *   The name of the user-controlled pref (relative to `browser.urlbar.`) that
+ *   controls the suggestion type. Should be the same as
+ *   `feature.primaryUserControlledPreference`.
+ * @param {Array} options.queries
+ *   Array of objects: `{ query, expectedResults }`
+ *   For each object, the test will perform a search with `query` as the search
+ *   string. After dismissing the suggestion type, the query shouldn't match any
+ *   results. After clearing dismissals, the query should match the results in
+ *   `expectedResults`. If `expectedResults` is omitted, `[result]` will be
+ *   used.
+ */
+async function doDismissAllTest({ feature, result, command, pref, queries }) {
+  await QuickSuggest.clearDismissedSuggestions();
+  await QuickSuggestTestUtils.forceSync();
+  Assert.ok(
+    !(await QuickSuggest.canClearDismissedSuggestions()),
+    "Sanity check: canClearDismissedSuggestions should return false initially"
+  );
+
+  let changedPromise = TestUtils.topicObserved(
+    "quicksuggest-dismissals-changed"
+  );
+
+  triggerCommand({
+    result,
+    command,
+    feature,
+    expectedCountsByCall: {
+      removeResult: 1,
+    },
+  });
+
+  info("Awaiting dismissals-changed promise");
+  await changedPromise;
+
+  Assert.ok(
+    await QuickSuggest.canClearDismissedSuggestions(),
+    "canClearDismissedSuggestions should return true after triggering command"
+  );
+  Assert.ok(
+    !UrlbarPrefs.get(pref),
+    "Pref should be false after triggering command: " + pref
+  );
+
+  for (let { query } of queries) {
+    info("Doing search after triggering command: " + JSON.stringify(query));
+    await check_results({
+      context: createContext(query, {
+        providers: [UrlbarProviderQuickSuggest.name],
+        isPrivate: false,
+      }),
+      matches: [],
+    });
+  }
+
+  let clearedPromise = TestUtils.topicObserved(
+    "quicksuggest-dismissals-cleared"
+  );
+
+  info("Clearing dismissals");
+  await QuickSuggest.clearDismissedSuggestions();
+
+  // It's not necessary to await this -- awaiting `clearDismissedSuggestions()`
+  // is sufficient -- but we do it to make sure the notification is sent.
+  info("Awaiting dismissals-cleared promise");
+  await clearedPromise;
+
+  Assert.ok(
+    !(await QuickSuggest.canClearDismissedSuggestions()),
+    "canClearDismissedSuggestions should return false after clearing dismissals"
+  );
+  Assert.ok(
+    UrlbarPrefs.get(pref),
+    "Pref should be true after clearing it: " + pref
+  );
+
+  // Clearing the pref will trigger a sync, so wait for it.
+  await QuickSuggestTestUtils.forceSync();
+
+  for (let { query, expectedResults = [result] } of queries) {
+    info("Doing search after clearing dismissals: " + JSON.stringify(query));
+    await check_results({
+      context: createContext(query, {
+        providers: [UrlbarProviderQuickSuggest.name],
+        isPrivate: false,
+      }),
+      matches: expectedResults,
+    });
+  }
 }
 
 /**
  * Does some "show less frequently" tests where the cap is set in remote
  * settings and Nimbus. See `doOneShowLessFrequentlyTest()`. This function
- * assumes the matching behavior implemented by the given `BaseFeature` is based
- * on matching prefixes of the given keyword starting at the first word. It
- * also assumes the `BaseFeature` provides suggestions in remote settings.
+ * assumes the matching behavior implemented by the given `SuggestFeature` is
+ * based on matching prefixes of the given keyword starting at the first word.
+ * It also assumes the `SuggestFeature` provides suggestions in remote settings.
  *
  * @param {object} options
  *   Options object.
- * @param {BaseFeature} options.feature
+ * @param {SuggestFeature} options.feature
  *   The feature that provides the suggestion matched by the searches.
  * @param {*} options.expectedResult
  *   The expected result that should be matched, for searches that are expected
@@ -773,7 +567,7 @@ async function doShowLessFrequentlyTests({
  *
  * @param {object} options
  *   Options object.
- * @param {BaseFeature} options.feature
+ * @param {SuggestFeature} options.feature
  *   The feature that provides the suggestion matched by the searches.
  * @param {*} options.expectedResult
  *   The expected result that should be matched, for searches that are expected
@@ -813,11 +607,11 @@ async function doOneShowLessFrequentlyTest({
   nimbus = {},
 }) {
   // Disable Merino so we trigger only remote settings suggestions. The
-  // `BaseFeature` is expected to add remote settings suggestions using keywords
-  // start starting with the first word in each full keyword, but the mock
-  // Merino server will always return whatever suggestion it's told to return
-  // regardless of the search string. That means Merino will return a suggestion
-  // for a keyword that's smaller than the first full word.
+  // `SuggestFeature` is expected to add remote settings suggestions using
+  // keywords start starting with the first word in each full keyword, but the
+  // mock Merino server will always return whatever suggestion it's told to
+  // return regardless of the search string. That means Merino will return a
+  // suggestion for a keyword that's smaller than the first full word.
   UrlbarPrefs.set("quicksuggest.dataCollection.enabled", false);
 
   // Set Nimbus variables and RS config.
@@ -923,8 +717,6 @@ async function doOneShowLessFrequentlyTest({
  *     The order doesn't matter.
  */
 async function doRustProvidersTests({ searchString, tests }) {
-  UrlbarPrefs.set("quicksuggest.rustEnabled", true);
-
   for (let { prefs, expectedUrls } of tests) {
     info(
       "Starting Rust providers test: " + JSON.stringify({ prefs, expectedUrls })
@@ -952,45 +744,76 @@ async function doRustProvidersTests({ searchString, tests }) {
     }
     await QuickSuggestTestUtils.forceSync();
   }
-
-  info("Clearing rustEnabled pref and forcing sync");
-  UrlbarPrefs.clear("quicksuggest.rustEnabled");
-  await QuickSuggestTestUtils.forceSync();
 }
 
 /**
- * Waits for `Weather` to start and finish a new fetch. Typically you call this
- * before you know a new fetch will start, save but don't await the promise, do
- * the thing that triggers the new fetch, and then await the promise to wait for
- * the fetch to finish.
+ * Simulates performing a command for a feature by calling its `onEngagement()`.
  *
- * If a fetch is currently ongoing, this will first wait for a new fetch to
- * start, which might not be what you want. If you only want to wait for any
- * ongoing fetch to finish, await `QuickSuggest.weather.fetchPromise` instead.
+ * @param {object} options
+ *   Options object.
+ * @param {SuggestFeature} options.feature
+ *   The feature whose command will be triggered.
+ * @param {string} options.command
+ *   The name of the command to trigger.
+ * @param {UrlbarResult} options.result
+ *   The result that the command will act on.
+ * @param {string} options.searchString
+ *   The search string to pass to `onEngagement()`.
+ * @param {object} options.expectedCountsByCall
+ *   If non-null, this should map controller and view method names to the number
+ *   of times they should be called in response to the command.
+ * @returns {Map}
+ *   A map from names of methods on the controller and view to the number of
+ *   times they were called.
  */
-async function waitForNewWeatherFetch() {
-  let { fetchPromise: oldFetchPromise } = QuickSuggest.weather;
+function triggerCommand({
+  feature,
+  command,
+  result,
+  searchString = "",
+  expectedCountsByCall = null,
+}) {
+  info(`Calling ${feature.name}.onEngagement() to trigger command: ${command}`);
 
-  // Wait for a new fetch to start.
-  let newFetchPromise;
-  await TestUtils.waitForCondition(() => {
-    let { fetchPromise } = QuickSuggest.weather;
-    if (
-      (oldFetchPromise && fetchPromise != oldFetchPromise) ||
-      (!oldFetchPromise && fetchPromise)
-    ) {
-      newFetchPromise = fetchPromise;
-      return true;
+  let countsByCall = new Map();
+  let addCall = name => {
+    if (!countsByCall.has(name)) {
+      countsByCall.set(name, 0);
     }
-    return false;
-  }, "Waiting for a new weather fetch to start");
+    countsByCall.set(name, countsByCall.get(name) + 1);
+  };
 
-  Assert.equal(
-    QuickSuggest.weather.fetchPromise,
-    newFetchPromise,
-    "Sanity check: fetchPromise hasn't changed since waitForCondition returned"
+  feature.onEngagement(
+    // query context
+    {},
+    // controller
+    {
+      removeResult() {
+        addCall("removeResult");
+      },
+      view: {
+        acknowledgeFeedback() {
+          addCall("acknowledgeFeedback");
+        },
+        invalidateResultMenuCommands() {
+          addCall("invalidateResultMenuCommands");
+        },
+      },
+    },
+    // details
+    { result, selType: command },
+    searchString
   );
 
-  // Wait for the new fetch to finish.
-  await newFetchPromise;
+  if (expectedCountsByCall) {
+    for (let [name, expectedCount] of Object.entries(expectedCountsByCall)) {
+      Assert.equal(
+        countsByCall.get(name) ?? 0,
+        expectedCount,
+        "Function should have been called the expected number of times: " + name
+      );
+    }
+  }
+
+  return countsByCall;
 }
